/******************************************************************************
 * DatabaseImpl.java - Implement the BuckoFIBS database API.
 * $Id: DatabaseImpl.java,v 1.8 2011/01/01 20:28:56 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: DatabaseImpl.java,v $
 * Revision 1.8  2011/01/01 20:28:56  dick
 * Add some debug for groups.
 *
 * Revision 1.7  2011/01/01 02:32:43  dick
 * Manging GroupOfPlayers and PlayerGroups satisfactorily.
 *
 * Revision 1.6  2011/01/01 01:26:57  dick
 * Load the PlayerGroups
 *
 * Revision 1.5  2010/12/31 05:29:25  dick
 * Add support for reading and writing GroupOfPlayers.
 *
 * Revision 1.4  2010/12/24 02:59:42  dick
 * WinLoss becomes a light object instead of a String.
 *
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:48  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.6  2010/01/29 23:07:08  dick
 * Remove aborted RatingGraphConfig, it's a properties object.
 *
 * Revision 1.5  2010/01/27 02:09:37  dick
 * Preliminary RatingGraphConfig work.
 *
 * Revision 1.4  2010/01/23 06:16:06  dick
 * Update finishedMatches for an opponent.
 *
 * Revision 1.3  2009/03/04 19:01:58  dick
 * Add getFinishedMatches(pid).
 *
 * Revision 1.2  2009/02/21 18:00:01  dick
 * Add getAllRatingsByDate()
 *
 * Revision 1.1  2009/02/20 10:24:33  dick
 * Implement the BuckoFIBS database API.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 */

package com.buckosoft.fibs.BuckoFIBS.db;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.hibernate.Session;

import com.buckosoft.fibs.domain.FinishedMatch;
import com.buckosoft.fibs.domain.GroupOfPlayers;
import com.buckosoft.fibs.domain.Player;
import com.buckosoft.fibs.domain.PlayerGroup;

/** Implement the BuckoFIBS database API.
 * @author Dick Balaska
 * @since 2009/02/18
 * @version $Revision: 1.8 $ <br> $Date: 2011/01/01 20:28:56 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/com/buckosoft/fibs/BuckoFIBS/db/DatabaseImpl.java">cvs DatabaseImpl.java</a>
 */
public class DatabaseImpl implements Database {
	private	final static boolean DEBUG = false;

	private boolean	virgin = true;
	
	private	List<FinishedMatch>	finishedMatches;


	public DatabaseImpl() {
	}

	private void initDB() {
		if (virgin)
			HibernateUtil.maybeCreateNewDatabase();
		virgin = false;
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#store(com.buckosoft.fibs.domain.Player)
	 */
	@Override
	public void store(Player player) {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		session.save(player);
		session.getTransaction().commit();

	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#getPlayer(int)
	 */
	@Override
	public Player getPlayer(int id) {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		Player p = null;
		try {
			p = (Player)session.createQuery("from Player where id = ?")
					.setInteger(0, id).uniqueResult();
		} catch (Exception e) {
			e.printStackTrace();
			return(null);
		}
		session.getTransaction().commit();
		return p;
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#getPlayer(java.lang.String)
	 */
	@Override
	public Player getPlayer(String name) {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		Player p = null;
		try {
			p = (Player)session.createQuery("from Player where name = ?")
					.setString(0, name).uniqueResult();
		} catch (Exception e) {
			e.printStackTrace();
			return(null);
		}
		session.getTransaction().commit();
		return p;
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#store(com.buckosoft.fibs.domain.FinishedMatch)
	 */
	@Override
	public void store(FinishedMatch finishedMatch) {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		session.save(finishedMatch);
		session.getTransaction().commit();
		this.finishedMatches = null;
	}

	@SuppressWarnings("unchecked")
	private	void getFinishedMatches() {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		List<?> list = session.createQuery("from FinishedMatch ORDER BY MatchDate").list();
		finishedMatches = (List<FinishedMatch>) list;
		session.getTransaction().commit();
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#getAllRatingsByDate()
	 */
	@Override
	public double[] getAllRatingsByDate() {
		if (this.finishedMatches == null)
			getFinishedMatches();
		double[] dd = new double[this.finishedMatches.size()];
		int i = 0;
		for (FinishedMatch fm : this.finishedMatches) {
			dd[i++] = fm.getRatingAfterMatch();
		}
		return dd;
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#updateOpponent(com.buckosoft.fibs.domain.Player)
	 */
	@Override
	public void updateOpponent(Player p) {
		if (this.finishedMatches == null)
			getFinishedMatches();
		if (p.getId() == 0) {
			Player p1 = this.getPlayer(p.getName());
			if (p1 != null)
				p.setId(p1.getId());
			else
				return;
		}
		int w = 0;
		int l = 0;
		for (FinishedMatch fm : this.finishedMatches) {
			if (fm.getOpponentId() == p.getId()) {
				if (fm.getYourScore() > fm.getOpponentScore())
					w++;
				else
					l++;
			}
		}
		if (w != 0 || l != 0)
			p.setWinLoss(w, l);
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#getFinishedMatches(int)
	 */
	@Override
	public List<FinishedMatch> getFinishedMatches(int pid) {
		List<FinishedMatch> matches = new LinkedList<FinishedMatch>();
		for (FinishedMatch fm : this.finishedMatches) {
			if (fm.getOpponentId() == pid)
				matches.add(fm);
		}
		return(matches);
	}

	@SuppressWarnings("unchecked")
	public	List<GroupOfPlayers> getGroupsOfPlayers() {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		List<?> list = session.createQuery("from GroupOfPlayers").list();
		List<GroupOfPlayers> ll = (List<GroupOfPlayers>) list;
		for (GroupOfPlayers gop : ll) {
			loadPlayerGroups(gop);
			gop.setDirty(false);
		}
		session.getTransaction().commit();
		return(ll);
	}

	@SuppressWarnings("unchecked")
	private void loadPlayerGroups(GroupOfPlayers groupOfPlayers) {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		List<?> list = session.createQuery("from PlayerGroup where groupId = ?").setInteger(0, groupOfPlayers.getId()).list();
		List<PlayerGroup> lpg = (List<PlayerGroup>)list;
		for (PlayerGroup pg : lpg) {
			pg.setDirty(false);
		}
		groupOfPlayers.setPlayerGroups(lpg);
	}

	public void store(List<GroupOfPlayers> groupList) {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		for (GroupOfPlayers gop : groupList) {
			if (gop.isDirty()) {
				store(gop);
				gop.setDirty(false);
			}
		}
		session.getTransaction().commit();
	}

	private void store(GroupOfPlayers groupOfPlayers) {
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		if (DEBUG)
			System.out.println("db: Store=" + groupOfPlayers.getGroupName());
		session.saveOrUpdate(groupOfPlayers);
		Iterator<PlayerGroup> iter = groupOfPlayers.getPlayerGroups().iterator();
		while (iter.hasNext()) {
			PlayerGroup pg = iter.next();
			if (pg.isTagForDelete()) {
				session.delete(pg);
				iter.remove();
			} else if (pg.isDirty()) {
				session.save(pg);
				pg.setDirty(false);
			}
		}

	}
}
