/******************************************************************************
 * InviterListPopupMenu.java - The context menu on the inviter table
 * $Id: InviterNameRenderer.java,v 1.3 2010/03/03 13:12:21 inim Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: InviterNameRenderer.java,v $
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.3  2009/02/14 15:46:36  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.2  2009/02/02 08:39:48  dick
 * Don't display the name in the default bold.
 *
 * Revision 1.1  2009/02/01 09:01:49  dick
 * PlayerNameRenderer becomes InviterNameRenderer in this package.
 *
 * Revision 1.1  2009/01/28 19:39:15  dick
 * package com.buckosoft.fibs.gui.inviterList becomes com.buckosoft.fibs.BuckoFIBS.gui.inviterList.
 *
 * Revision 1.1  2009/01/27 19:16:53  dick
 * playerName gets a custom renderer so we can display warnings.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.inviterList;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import com.buckosoft.fibs.domain.Player;

/** Draw the player name and any icons in the inviter table
 * @author Dick Balaska
 * @since 2009/01/27
 * @version $Revision: 1.3 $ <br> $Date: 2010/03/03 13:12:21 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/inviterList/InviterNameRenderer.java">cvs InviterNameRenderer.java</a>
 */
public class InviterNameRenderer extends JPanel implements TableCellRenderer {
	private static final long serialVersionUID = 1L;
	private	Color	selectedBackgroundColor;
	private JLabel jLabel = null;
	private JLabel jLabelIcon = null;
	private	ImageIcon	warningIcon = new ImageIcon(getClass().getResource("/g/caution.png"));

	/**
	 * This method initializes 
	 * 
	 */
	public InviterNameRenderer() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.insets = new Insets(0, 0, 0, 0);
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.gridy = 0;
		gridBagConstraints1.anchor = GridBagConstraints.EAST;
		gridBagConstraints1.weightx = 0.2;
		gridBagConstraints1.gridheight = 1;
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.insets = new Insets(0, 0, 0, 0);
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagConstraints.gridheight = 1;
		jLabelIcon = new JLabel();
		jLabel = new JLabel();
		jLabel.setFont(new Font("Dialog", Font.PLAIN, 12));
		jLabel.setText("JLabel");
		this.setLayout(new GridBagLayout());
		this.setSize(new Dimension(79, 22));
		this.add(jLabel, gridBagConstraints);
		this.add(jLabelIcon, gridBagConstraints1);
	}

	public void setSelectedBackground(Color color) {
		selectedBackgroundColor = color;
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
	 */
	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		Player p = (Player)value;
		this.setOpaque(true);
		this.jLabel.setOpaque(true);
		if (isSelected) {
			this.setBackground(selectedBackgroundColor);
			this.jLabel.setBackground(selectedBackgroundColor);
		} else {
			this.setBackground(Color.white);
			this.jLabel.setBackground(Color.white);
		}
		this.jLabel.setText(p.getName());
		this.jLabel.setOpaque(true);
		if (p.getBfStatus() != null && p.getBfStatus().length() > 0) {
			this.jLabelIcon.setIcon(warningIcon);
			this.setToolTipText(p.getBfStatus());
		} else {
			this.jLabelIcon.setIcon(null);
			this.setToolTipText(null);
			
		}
		return this;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
