/******************************************************************************
 * PlayerListPane.java - Display the list of players in a panel with buttons and scrollbars and everything.
 * $Id: PlayerListPane.java,v 1.8 2011/01/01 20:45:50 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: PlayerListPane.java,v $
 * Revision 1.8  2011/01/01 20:45:50  dick
 * Update the group text box on changes.
 *
 * Revision 1.7  2011/01/01 06:11:55  dick
 * Run the filters through the GroupManager.
 *
 * Revision 1.6  2011/01/01 01:26:24  dick
 * Rebuild the group menu when displaying the popup.
 *
 * Revision 1.5  2010/12/31 05:30:58  dick
 * Add support for displaying the GroupSelectPane and friends.
 *
 * Revision 1.4  2010/12/30 04:10:51  dick
 * Bug 8 - Shouldn't be able to invite players that are already playing.
 *
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.12  2010/01/25 02:54:43  dick
 * Use the selected index at the time the right mouse was clicked.
 *
 * Revision 1.11  2010/01/24 06:21:36  dick
 * Don't update the playerList if the popupmenu is open, it gets confused.
 *
 * Revision 1.10  2010/01/23 07:48:49  dick
 * Turn off youInvited when a player's state changed.
 *
 * Revision 1.9  2010/01/23 06:25:20  dick
 * Add repbot and playerInvited handling.
 *
 * Revision 1.8  2009/03/12 16:36:17  dick
 * Move the double-click test above, so we check for a valid player.
 *
 * Revision 1.7  2009/03/04 19:04:45  dick
 * Get the row filter working.
 * If you doubleclick on a player, then generate a player report request.
 *
 * Revision 1.6  2009/03/01 09:11:56  dick
 * ROP filtering is functional.
 *
 * Revision 1.5  2009/02/24 08:05:42  dick
 * Try filtering rows based on ROP.  DDS.
 *
 * Revision 1.4  2009/02/17 14:44:50  dick
 * Turn off DEBUG.
 *
 * Revision 1.3  2009/02/14 15:46:36  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.2  2009/02/01 21:28:00  dick
 * Add the invite submenu.
 *
 * Revision 1.1  2009/01/28 19:39:44  dick
 * package com.buckosoft.fibs.gui.playerList becomes com.buckosoft.fibs.BuckoFIBS.gui.playerList.
 *
 * Revision 1.7  2009/01/22 05:06:30  dick
 * Add sorting and variable column widths.
 *
 * Revision 1.6  2008/12/15 01:50:53  dick
 * Display the number of players in the list.
 *
 * Revision 1.5  2008/12/12 14:46:49  dick
 * Handle the Watch command.
 *
 * Revision 1.4  2008/12/11 20:34:36  dick
 * Javadoc.
 *
 * Revision 1.3  2008/04/09 20:58:04  dick
 * Handle inviting a player to a match.
 *
 * Revision 1.2  2008/04/07 08:12:33  dick
 * Add the popup menu to the PlayerList.
 *
 * Revision 1.1  2008/04/02 23:22:53  dick
 * Move playerList to it's own package.
 *
 * Revision 1.1  2008/04/01 03:45:14  dick
 * Display the list of players in a panel with buttons and scrollbars and everything.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.playerList;

import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.RowFilter;
import javax.swing.table.TableRowSorter;

import com.buckosoft.fibs.BuckoFIBS.CommandDispatcher;
import com.buckosoft.fibs.BuckoFIBS.GroupManager;
import com.buckosoft.fibs.BuckoFIBS.gui.MainDialog;
import com.buckosoft.fibs.BuckoFIBS.gui.playerList.group.GroupSelectPane;
import com.buckosoft.fibs.domain.Player;
import java.awt.Font;

/** Display the list of Players
 * @author Dick Balaska
 * @since 2008/04/01
 * @version $Revision: 1.8 $ <br> $Date: 2011/01/01 20:45:50 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/main/java/com/buckosoft/fibs/BuckoFIBS/gui/playerList/PlayerListPane.java">cvs PlayerListPane.java</a>
 */
public class PlayerListPane extends JPanel implements ActionListener {
	private	final static boolean DEBUG = false;
	private static final long serialVersionUID = 1L;
	private JToolBar jToolBar = null;
	private JScrollPane playerListScrollPane = null;
	private JToggleButton readyToggleButton = null;
	private JToggleButton onlineToggleButton = null;
	private JToggleButton playingToggleButton = null;

	private JTable			playerListTable = null;
	private	PlayerTableModel playerTableModel;

	private	PlayerListPopupMenu playerListPopupMenu = null;
	private	CommandDispatcher	commandDispatcher = null;
	private	GroupManager		groupManager;
	private GroupSelectPane		groupPane;
	private	MainDialog			mainDialog;

	/** Count of the number of players logged in */
	private JLabel jLabelLoggedIn = null;

	/** Filter on Ready/Online/Playing */
	private	boolean[] rop = new boolean[] {false, false, false};

	private	boolean	updateLocked = false;
	private	int		selectedIndex = -2;

	/** This is the default constructor
	 */
	public PlayerListPane() {
		super();
		initialize();
	}

	public PlayerTableModel getPlayerTableModel() {
		return(playerTableModel);
	}

	public void setCommandDispatcher(CommandDispatcher commandDispatcher) {
		this.commandDispatcher = commandDispatcher;
	}

	public void setGroupManager(GroupManager groupManager) {
		this.groupManager = groupManager;
		playerListPopupMenu.setGroupManager(groupManager);
		this.groupManager.setPlayerListPane(this);
		groupPaneChanged();
	}

	public void setMainDialog(MainDialog mainDialog) {
		this.mainDialog = mainDialog;
	}

	/** Return the Player in the table that matches this name.
	 * @param playerName The name to query
	 * @return The Player found or null if no match
	 */
	public Player getPlayer(String playerName) {
		return(this.playerTableModel.getPlayer(playerName));
	}

	public void playerChanged(Player player) {
		this.playerTableModel.playerChanged(player);
		if (!this.updateLocked) {
			updatePlayerCount();
			playerListTable.setRowSorter(sorter);
			sorter.setRowFilter(rowFilter);
		}
	}

	public void playerGone(String playerName) {
		this.playerTableModel.playerGone(playerName);
		updatePlayerCount();
	}

	/** Turn on the "you invited" icon for a player.
	 * Note that you only get one open invite. 
	 * @param playerName
	 */
	public void playerInvited(String playerName) {
		if (playerName == null)
			playerUninvited();
		else {
			this.playerTableModel.playerInvited(playerName);
			if (!this.updateLocked) {
				updatePlayerCount();
				playerListTable.setRowSorter(sorter);
				sorter.setRowFilter(rowFilter);
			}
		}
	}

	private void playerUninvited() {
		if (this.playerTableModel.playerUninvited()) {
			if (!this.updateLocked) {
				updatePlayerCount();
				playerListTable.setRowSorter(sorter);
				sorter.setRowFilter(rowFilter);
			}
		}
	}
	/** Set the states of the Ready, Online, and Playing buttons.
	 * @param rop The state of the Ready, Online, and Playing buttons.
	 */
	public void setROP(boolean[] rop) {
		this.readyToggleButton.setSelected(rop[0]);
		this.onlineToggleButton.setSelected(rop[1]);
		this.playingToggleButton.setSelected(rop[2]);
		this.rop = rop;
		playerListTable.setRowSorter(sorter);
		sorter.setRowFilter(rowFilter);
		sorter.sort();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		getPlayerListPopupMenu();
		GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
		gridBagConstraints5.gridx = 1;
		gridBagConstraints5.weightx = 0.1;
		gridBagConstraints5.gridy = 0;
		GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
		gridBagConstraints11.gridx = 2;
		gridBagConstraints11.insets = new Insets(0, 0, 0, 13);
		gridBagConstraints11.weightx = 0.2;
		gridBagConstraints11.gridy = 0;
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.fill = GridBagConstraints.BOTH;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.weightx = 1.0;
		gridBagConstraints1.weighty = 1.0;
		gridBagConstraints1.gridwidth = 3;
		gridBagConstraints1.gridx = 0;
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagConstraints.gridx = 0;
		this.setSize(300, 200);
		this.setLayout(new GridBagLayout());
		this.add(getJToolBar(), gridBagConstraints);
		jLabelLoggedIn = new JLabel();
		jLabelLoggedIn.setText("On:");
		this.add(getPlayerListScrollPane(), gridBagConstraints1);
		this.add(getJPanelDummyAfterGroup(), gridBagConstraints5);
		this.add(jLabelLoggedIn, gridBagConstraints11);
	}

	
	/** Called directly from the menu.
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals(PlayerListPopupMenu.s_setVisible)) {
			this.updateLocked = e.getModifiers() == 0 ? false : true;
			if (!this.updateLocked) {
				updatePlayerCount();
				playerListTable.setRowSorter(sorter);
				sorter.setRowFilter(rowFilter);
			} else {
				this.selectedIndex = playerListTable.getSelectedRow();
			}
			return;
		}
		if (DEBUG)
			System.out.println("playerListTable.getSelectedRow() = " + playerListTable.getSelectedRow());
//		Player p = (Player)playerListTable.getModel().getValueAt(playerListTable.convertRowIndexToModel(playerListTable.getSelectedRow()), 0);
		Player p = (Player)playerListTable.getModel().getValueAt(playerListTable.convertRowIndexToModel(this.selectedIndex), 0);
		String playerName = p.getName(); 
		if (DEBUG)
			System.out.println("Action performed: " + e.getActionCommand() + " on " + playerName);
		if (e.getActionCommand().startsWith("inv-")) {
			String s = e.getActionCommand().substring(4);
			this.commandDispatcher.dispatch(CommandDispatcher.Command.INVITE, playerName, s);
		} else if (e.getActionCommand() == "Watch") {
			this.commandDispatcher.dispatch(CommandDispatcher.Command.WATCH, playerName);
		} else if (e.getActionCommand().startsWith("rep")) {
			String s = "tell repbot " + e.getActionCommand().substring(3) + " " + playerName;
			this.commandDispatcher.dispatch(CommandDispatcher.Command.SEND_COMMAND, s);
			
		} else {
			System.out.println("Unhandled Action: " + e.getActionCommand() + " on " + playerName);			
		}
		
	}

	private	void	onMousePressed(MouseEvent e) {
		if (e.getButton() != MouseEvent.BUTTON1)
			return;
		if (e.getClickCount() != 2)
			return;
		Player p = (Player)playerListTable.getModel().getValueAt(playerListTable.convertRowIndexToModel(playerListTable.getSelectedRow()), 0);
		if (p == null)
			return;
		String playerName = p.getName();
		if (DEBUG)
			System.out.println("dblclick performed: on " + playerName);
		this.commandDispatcher.dispatch(CommandDispatcher.Command.GET_PLAYER_REPORT, playerName);
	}

	/**
	 * This method initializes jToolBar	
	 * 	
	 * @return javax.swing.JToolBar	
	 */
	private JToolBar getJToolBar() {
		if (jToolBar == null) {
			jToolBar = new JToolBar();
			jToolBar.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			jToolBar.setFloatable(false);
			jToolBar.add(getReadyToggleButton());
			jToolBar.add(getOnlineToggleButton());
			jToolBar.add(getPlayingToggleButton());
			jToolBar.add(new JToolBar.Separator());
			jToolBar.add(getJTextFieldGroup());
		}
		return jToolBar;
	}

	/**
	 * This method initializes playerListScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getPlayerListScrollPane() {
		if (playerListScrollPane == null) {
			playerListScrollPane = new JScrollPane();
			playerListScrollPane.setViewportView(getPlayerListTable());
		}
		return playerListScrollPane;
	}
	
	private TableRowSorter<PlayerTableModel> sorter;
	RowFilter<PlayerTableModel,Integer> rowFilter = new RowFilter<PlayerTableModel,Integer>() {
		public boolean include(Entry<? extends PlayerTableModel, ? extends Integer> entry) {
			PlayerTableModel playerModel = entry.getModel();
			Player player = (Player)playerModel.getValueAt(entry.getIdentifier(), 0);
			if (!groupManager.filter(player))
				return(false);
			if (player.isPlaying())
					return(rop[2]);
			if (player.isReady())
				return(rop[0]);
			return(rop[1]);
		}
	};
	private JTextField jTextFieldGroup = null;
	private JPanel jPanelDummyAfterGroup = null;
	
	/** This method initializes playerListTable	
	 * @return javax.swing.JTable	
	 */
	private JTable getPlayerListTable() {
		if (playerListTable == null) {
			playerTableModel = new PlayerTableModel();
			sorter = new TableRowSorter<PlayerTableModel>(playerTableModel);
			playerListTable = new JTable(playerTableModel);
			playerListTable.addMouseListener(new java.awt.event.MouseAdapter() {   
				public void mousePressed(java.awt.event.MouseEvent e) {    
					onMousePressed(e);
				}
			});
			playerListTable.setRowSorter(sorter);
			sorter.setRowFilter(rowFilter);
			playerListTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
			for (int i=0; i<playerTableModel.getColumnCount(); i++) {
				int w = playerTableModel.getColumnWidth(i);
				playerListTable.getColumnModel().getColumn(i).setPreferredWidth(w);
			}
			playerListTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			playerListTable.setAutoCreateRowSorter(true);

			PlayerNameRenderer playerRenderer = new PlayerNameRenderer();
			playerListTable.setDefaultRenderer(com.buckosoft.fibs.domain.Player.class, playerRenderer);
			playerRenderer.setBackground(playerListTable.getBackground());
			playerRenderer.setSelectedBackground(playerListTable.getSelectionBackground());

			//Add listener to components that can bring up popup menus.
			PopupListener popupListener = new PopupListener();
			popupListener.setPlayerListPane(this);
			playerListTable.addMouseListener(popupListener);
		}
		return playerListTable;
	}
	/**
	 * This method initializes readyToggleButton	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	private JToggleButton getReadyToggleButton() {
		if (readyToggleButton == null) {
			readyToggleButton = new JToggleButton();
			readyToggleButton.setIcon(new ImageIcon(getClass().getResource("/g/pready.gif")));
			readyToggleButton.setSelected(true);
			readyToggleButton.setMnemonic(KeyEvent.VK_UNDEFINED);
			readyToggleButton.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					changeROP();
				}
			});
		}
		return readyToggleButton;
	}

	/**
	 * This method initializes onlineToggleButton	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	private JToggleButton getOnlineToggleButton() {
		if (onlineToggleButton == null) {
			onlineToggleButton = new JToggleButton();
			onlineToggleButton.setIcon(new ImageIcon(getClass().getResource("/g/ponline.gif")));
			onlineToggleButton.setSelected(true);
			onlineToggleButton.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					changeROP();
				}
			});
		}
		return onlineToggleButton;
	}

	/**
	 * This method initializes playingToggleButton	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	private JToggleButton getPlayingToggleButton() {
		if (playingToggleButton == null) {
			playingToggleButton = new JToggleButton();
			playingToggleButton.setIcon(new ImageIcon(getClass().getResource("/g/pplaying.gif")));
			playingToggleButton.setSelected(true);
			playingToggleButton.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					changeROP();
				}
			});
		}
		return playingToggleButton;
	}

	private	void changeROP() {
		boolean[] rop = new boolean[3];
		rop[0] = this.readyToggleButton.getModel().isSelected();
		rop[1] = this.onlineToggleButton.getModel().isSelected();
		rop[2] = this.playingToggleButton.getModel().isSelected();
		this.commandDispatcher.ropChanged(rop);
	}

	private	PlayerListPopupMenu getPlayerListPopupMenu() { 
		if (playerListPopupMenu == null) {
			playerListPopupMenu = new PlayerListPopupMenu();
			playerListPopupMenu.setMenuHandler(this);
		}
		return(playerListPopupMenu);
	}

	private void updatePlayerCount() {
		String s = "On: " + this.playerTableModel.getRowCount();
		this.jLabelLoggedIn.setText(s);
	}

	class PopupListener extends MouseAdapter {
		PlayerListPane	plp;
		
		void setPlayerListPane(PlayerListPane plp) {
			this.plp = plp;
		}
		public void mousePressed(MouseEvent e) {
			maybeShowPopup(e);
		}

		public void mouseReleased(MouseEvent e) {
			maybeShowPopup(e);
		}

		private void maybeShowPopup(MouseEvent e) {
			if (e.isPopupTrigger()) {
				int row = playerListTable.rowAtPoint(new Point(e.getX(), e.getY()));
				if (row != -1)
					playerListTable.getSelectionModel().setSelectionInterval(row, row);
				//if (DEBUG)
				//	System.out.println("selected row = " + row);
				Player p = (Player)this.plp.playerTableModel.getValueAt(playerListTable.convertRowIndexToModel(playerListTable.getSelectedRow()), 0);
				playerListPopupMenu.setPlayer(p);
				playerListPopupMenu.rebuildGroups();
				playerListPopupMenu.getInvitePopupSubmenu().getJMenuItemResume().setVisible(
						p.getSavedMatch() != null);
				playerListPopupMenu.getInvitePopupSubmenu().setEnabled(p.isReady());
				playerListPopupMenu.show(e.getComponent(),
						e.getX(), e.getY());
			}
		}
	}

	/**
	 * This method initializes jTextFieldGroup	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextFieldGroup() {
		if (jTextFieldGroup == null) {
			jTextFieldGroup = new JTextField();
			jTextFieldGroup.setPreferredSize(new Dimension(140, 20));
			jTextFieldGroup.setEnabled(true);
			jTextFieldGroup.setFont(new Font("Dialog", Font.PLAIN, 14));
			jTextFieldGroup.setEditable(false);
			jTextFieldGroup.addMouseListener(new java.awt.event.MouseAdapter() {
				public void mouseClicked(java.awt.event.MouseEvent e) {
					handleGroupPane();
				}
			});
		}
		return jTextFieldGroup;
	}

	/**
	 * This method initializes jPanelDummyAfterGroup	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelDummyAfterGroup() {
		if (jPanelDummyAfterGroup == null) {
			jPanelDummyAfterGroup = new JPanel();
			jPanelDummyAfterGroup.setLayout(new GridBagLayout());
			jPanelDummyAfterGroup.setPreferredSize(new Dimension(20, 20));
		}
		return jPanelDummyAfterGroup;
	}

	private void handleGroupPane() {
		if (groupPane == null) {
			groupPane = createGroupPane();
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridheight = 2;
			gridBagConstraints.anchor = GridBagConstraints.NORTH;
			this.add(groupPane, gridBagConstraints);
			groupPane.setVisible(true);
			this.revalidate();
		} else {
			groupPane.setVisible(false);
			groupPane.savePaneToList();
			groupPane = null;
			this.groupManager.save();
		}
	}
	/** GroupManager calls here when one of the groups reports it has changed
	 */
	public void groupPaneChanged() {
		playerListTable.setRowSorter(sorter);
		sorter.setRowFilter(rowFilter);
		sorter.sort();
		getJTextFieldGroup().setText(this.groupManager.getGroupsSelectedString());
	}

	private GroupSelectPane createGroupPane() {
		GroupSelectPane gsp = new GroupSelectPane();
		gsp.setGroupManager(this.groupManager);
		gsp.setMainDialog(this.mainDialog);
		gsp.populatePane();
		return(gsp);
	}
}
