/******************************************************************************
 * Line.java - A single line of text from FIBS
 * $Id: Line.java,v 1.3 2010/03/03 13:12:21 inim Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Line.java,v $
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:48  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.4  2009/02/17 14:45:09  dick
 * Javadoc.
 *
 * Revision 1.3  2009/02/14 15:41:57  dick
 * BuckoFIBS is released under the GNU license.
 * Javadoc.
 *
 * Revision 1.2  2009/01/31 08:53:30  dick
 * Store the number of checkers to move with this Line.
 *
 * Revision 1.1  2009/01/31 06:02:18  dick
 * A single line of text from FIBS.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.domain;

/** A single line of text from FIBS. <br>
 * The only line we really care about is 'board' which is used by the GameManager
 * to log game moves. 
 * @author Dick Balaska
 * @since 2009/01/30
 * @version $Revision: 1.3 $ <br> $Date: 2010/03/03 13:12:21 $
 * @see com.buckosoft.fibs.BuckoFIBS.GameManager
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/domain/Line.java">cvs Line.java</a>
 */
public class Line {
	private int		cookie;
	private String	line;
	private	int		checkersToMove;

	/** Create an empty Line
	 */
	public Line() {}
	
	/** Create a line with these values.
	 * @param cookie The FIBSMessages cookie that was parsed.
	 * @param line The line of text received from FIBS
	 */
	public Line(int cookie, String line) {
		this.cookie = cookie;
		this.line = line;
	}

	/** Get the cookie of this Line.
	 * @see com.buckosoft.fibs.net.CookieMonster
	 * @return the cookie
	 */
	public int getCookie() {
		return cookie;
	}

	/** Set the cookie of this Line
	 * @param cookie the cookie to set
	 */
	public void setCookie(int cookie) {
		this.cookie = cookie;
	}

	/** Return the text of this Line
	 * @return the line
	 */
	public String getLine() {
		return line;
	}

	/** Set the text of this Line
	 * @param line the line to set
	 */
	public void setLine(String line) {
		this.line = line;
	}

	/** If this board is your move, then this is the number of checkers to move.
	 * If it is not your move, this value is 0.
	 * This is only valid if you are playing, not watching.
	 * @return the diceToMove
	 */
	public int getCheckersToMove() {
		return checkersToMove;
	}
	/** Set the number of checkers to move on this board.
	 * @param diceToMove the diceToMove to set
	 */
	public void setCheckersToMove(int diceToMove) {
		this.checkersToMove = diceToMove;
	}
}
